--Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
-- This script belongs to GIANTS Software and is used with their authorization.
-- This is an updated version of the "CutterAreaEvent.lua" file to allow the mod to handle the fruit "sugarbeet".
-- I do NOT own any rights for this script.

SugarBeetCutterAreaEvent = {};
SugarBeetCutterAreaEvent_mt = Class(SugarBeetCutterAreaEvent, Event);

InitEventClass(SugarBeetCutterAreaEvent, "SugarBeetCutterAreaEvent");

function SugarBeetCutterAreaEvent:emptyNew()
    local self = Event:new(SugarBeetCutterAreaEvent_mt);
    self.className="SugarBeetCutterAreaEvent";
    return self;
end;

function SugarBeetCutterAreaEvent:new(cuttingAreas, fruitType)
    local self = SugarBeetCutterAreaEvent:emptyNew()
    self.cuttingAreas = cuttingAreas;
    self.fruitType = fruitType;
    return self;
end;
 
function SugarBeetCutterAreaEvent:readStream(streamId, connection)
    local numAreas = streamReadUIntN(streamId, 4);
    local fruitType = streamReadInt8(streamId);
    
    local refX = streamReadFloat32(streamId);
    local refY = streamReadFloat32(streamId);
    local values = Utils.readCompressed2DVectors(streamId, refX, refY, numAreas*3-1, 0.01, true);
    for i=1, numAreas do
        local vi = i-1;
        local x = values[vi*3+1].x;
        local z = values[vi*3+1].y;
        local x1 = values[vi*3+2].x;
        local z1 = values[vi*3+2].y;
        local x2 = values[vi*3+3].x;
        local z2 = values[vi*3+3].y;
        Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
       -- Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2,0);
    end;
end;
  
function SugarBeetCutterAreaEvent:writeStream(streamId, connection)
    local numAreas = table.getn(self.cuttingAreas);
    streamWriteUIntN(streamId, numAreas, 4);
    streamWriteInt8(streamId, self.fruitType);
    local refX, refY;
    local values = {};
    for i=1, numAreas do
        local d = self.cuttingAreas[i];
        if i==1 then
            refX = d[1];
            refY = d[2];
            streamWriteFloat32(streamId, d[1]);
            streamWriteFloat32(streamId, d[2]);
        else
            table.insert(values, {x=d[1], y=d[2]});
        end;
        table.insert(values, {x=d[3], y=d[4]});
        table.insert(values, {x=d[5], y=d[6]});
    end;
    assert(table.getn(values) == numAreas*3 - 1);
    Utils.writeCompressed2DVectors(streamId, refX, refY, values, 0.01);
end;

function SugarBeetCutterAreaEvent:run(connection)
    print("Error: Do not run SugarBeetCutterAreaEvent locally");
end;

function SugarBeetCutterAreaEvent.runLocally(cuttingAreas, fruitType)
    
    local numAreas = table.getn(cuttingAreas);

    local refX, refY;
    local values = {};
    for i=1, numAreas do
        local d = cuttingAreas[i];
        if i==1 then
            refX = d[1];
            refY = d[2];
        else
            table.insert(values, {x=d[1], y=d[2]});
        end;
        table.insert(values, {x=d[3], y=d[4]});
        table.insert(values, {x=d[5], y=d[6]});
    end;
    assert(table.getn(values) == numAreas*3 - 1);

    local values = Utils.simWriteCompressed2DVectors(refX, refY, values, 0.01, true);

    local lastArea = 0;
    local realArea = 0;
    for i=1, numAreas do
        local vi = i-1;
        local x = values[vi*3+1].x;
        local z = values[vi*3+1].y;
        local x1 = values[vi*3+2].x;
        local z1 = values[vi*3+2].y;
        local x2 = values[vi*3+3].x;
        local z2 = values[vi*3+3].y;
        --Utils.updateFruitCutShortArea(fruitType, x, z, x1, z1, x2, z2, 1);
       -- Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2,0);
		
		local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2,0)*g_currentMission.windrowCutLongRatio;
		area = area + Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2,0);
        if area > 0 then

            local spray = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, x, z, x1, z1, x2, z2);
            local multi = 1;
            if spray > 0 then
                multi = 2;
            end;
            lastArea = lastArea + area*multi;
            realArea = realArea + area/g_currentMission.maxFruitValue;
        end;
    end;
    return lastArea, realArea;
end;